@@ -1,35 +1,21 @@
 /*
  * Copyright (c) 1999-2003 Todd C. Miller <Todd.Miller@courtesan.com>
- * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ * Permission to use, copy, modify, and distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
  *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
+ * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
+ * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
+ * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+ * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+ * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *
- * 3. The name of the author may not be used to endorse or promote products
- *    derived from this software without specific prior written permission.
- *
- * 4. Products derived from this software may not be called "Sudo" nor
- *    may "Sudo" appear in their names without specific prior written
- *    permission from the author.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
- * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
- * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
- * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
- * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
- * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
- * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
- * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * Sponsored in part by the Defense Advanced Research Projects
+ * Agency (DARPA) and Air Force Research Laboratory, Air Force
+ * Materiel Command, USAF, under agreement number F39502-99-1-0512.
  */
 
 #include "config.h"
@@ -55,12 +41,16 @@
 #if defined(HAVE_MALLOC_H) && !defined(STDC_HEADERS)
 # include <malloc.h>
 #endif /* HAVE_MALLOC_H && !STDC_HEADERS */
-#include <limits.h>
+#ifdef HAVE_ERR_H
+# include <err.h>
+#else
+# include "emul/err.h"
+#endif /* HAVE_ERR_H */
 
 #include "sudo.h"
 
 #ifndef lint
-static const char rcsid[] = "$Sudo: alloc.c,v 1.18 2003/03/15 20:31:01 millert Exp $";
+static const char rcsid[] = "$Sudo: alloc.c,v 1.23 2004/06/01 16:23:32 millert Exp $";
 #endif /* lint */
 
 /*
@@ -73,16 +63,10 @@ static const char rcsid[] = "$Sudo: alloc.c,v 1.18 2003/03/15 20:31:01 millert E
 # ifdef SIZE_T_MAX
 #  define SIZE_MAX	SIZE_T_MAX
 # else
-#  ifdef INT_MAX
-#   define SIZE_MAX	INT_MAX
-#  else
-#   define SIZE_MAX	0x7fffffff
-#  endif /* ULONG_MAX */
+#  define SIZE_MAX	INT_MAX
 # endif /* SIZE_T_MAX */
 #endif /* SIZE_MAX */
 
-extern char **Argv;		/* from sudo.c */
-
 /*
  * emalloc() calls the system malloc(3) and exits with an error if
  * malloc(3) fails.
@@ -93,15 +77,11 @@ emalloc(size)
 {
     VOID *ptr;
 
-    if (size == 0) {
-	(void) fprintf(stderr, "%s: internal error, tried to emalloc(0)\n",
-	    Argv[0]);
-	exit(1);
-    }
-    if ((ptr = (VOID *) malloc(size)) == NULL) {
-	(void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
-	exit(1);
-    }
+    if (size == 0)
+	errx(1, "internal error, tried to emalloc(0)");
+
+    if ((ptr = (VOID *) malloc(size)) == NULL)
+	errx(1, "unable to allocate memory");
     return(ptr);
 }
 
@@ -116,21 +96,14 @@ emalloc2(nmemb, size)
 {
     VOID *ptr;
 
-    if (nmemb == 0 || size == 0) {
-	(void) fprintf(stderr, "%s: internal error, tried to emalloc2(0)\n",
-	    Argv[0]);
-	exit(1);
-    }
-    if (nmemb > SIZE_MAX / size) {
-	(void) fprintf(stderr, "%s: internal error, emalloc2() overflow\n",
-	    Argv[0]);
-	exit(1);
-    }
+    if (nmemb == 0 || size == 0)
+	errx(1, "internal error, tried to emalloc2(0)");
+    if (nmemb > SIZE_MAX / size)
+	errx(1, "internal error, emalloc2() overflow");
+
     size *= nmemb;
-    if ((ptr = (VOID *) malloc(size)) == NULL) {
-	(void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
-	exit(1);
-    }
+    if ((ptr = (VOID *) malloc(size)) == NULL)
+	errx(1, "unable to allocate memory");
     return(ptr);
 }
 
@@ -145,16 +118,12 @@ erealloc(ptr, size)
     size_t size;
 {
 
-    if (size == 0) {
-	(void) fprintf(stderr, "%s: internal error, tried to erealloc(0)\n",
-	    Argv[0]);
-	exit(1);
-    }
+    if (size == 0)
+	errx(1, "internal error, tried to erealloc(0)");
+
     ptr = ptr ? (VOID *) realloc(ptr, size) : (VOID *) malloc(size);
-    if (ptr == NULL) {
-	(void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
-	exit(1);
-    }
+    if (ptr == NULL)
+	errx(1, "unable to allocate memory");
     return(ptr);
 }
 
@@ -171,22 +140,15 @@ erealloc3(ptr, nmemb, size)
     size_t size;
 {
 
-    if (nmemb == 0 || size == 0) {
-	(void) fprintf(stderr, "%s: internal error, tried to erealloc3(0)\n",
-	    Argv[0]);
-	exit(1);
-    }
-    if (nmemb > SIZE_MAX / size) {
-	(void) fprintf(stderr, "%s: internal error, erealloc3() overflow\n",
-	    Argv[0]);
-	exit(1);
-    }
+    if (nmemb == 0 || size == 0)
+	errx(1, "internal error, tried to erealloc3(0)");
+    if (nmemb > SIZE_MAX / size)
+	errx(1, "internal error, erealloc3() overflow");
+
     size *= nmemb;
     ptr = ptr ? (VOID *) realloc(ptr, size) : (VOID *) malloc(size);
-    if (ptr == NULL) {
-	(void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
-	exit(1);
-    }
+    if (ptr == NULL)
+	errx(1, "unable to allocate memory");
     return(ptr);
 }
 
@@ -236,10 +198,8 @@ easprintf(va_alist)
     len = vasprintf(ret, fmt, ap);
     va_end(ap);
 
-    if (len == -1) {
-	(void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
-	exit(1);
-    }
+    if (len == -1)
+	errx(1, "unable to allocate memory");
     return(len);
 }
 
@@ -255,9 +215,7 @@ evasprintf(ret, format, args)
 {
     int len;
 
-    if ((len = vasprintf(ret, format, args)) == -1) {
-	(void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
-	exit(1);
-    }
+    if ((len = vasprintf(ret, format, args)) == -1)
+	errx(1, "unable to allocate memory");
     return(len);
 }
