@@ -1,35 +1,21 @@
 /*
- * Copyright (c) 1999-2002 Todd C. Miller <Todd.Miller@courtesan.com>
- * All rights reserved.
+ * Copyright (c) 1999-2004 Todd C. Miller <Todd.Miller@courtesan.com>
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ * Permission to use, copy, modify, and distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
  *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
+ * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
+ * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
+ * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+ * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+ * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *
- * 3. The name of the author may not be used to endorse or promote products
- *    derived from this software without specific prior written permission.
- *
- * 4. Products derived from this software may not be called "Sudo" nor
- *    may "Sudo" appear in their names without specific prior written
- *    permission from the author.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
- * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
- * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
- * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
- * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
- * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
- * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
- * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * Sponsored in part by the Defense Advanced Research Projects
+ * Agency (DARPA) and Air Force Research Laboratory, Air Force
+ * Materiel Command, USAF, under agreement number F39502-99-1-0512.
  */
 
 #include "config.h"
@@ -60,13 +46,24 @@
 #endif /* HAVE_UNISTD_H */
 #include <pwd.h>
 
-#include <security/pam_appl.h>
+#ifdef HAVE_PAM_PAM_APPL_H
+# include <pam/pam_appl.h>
+#else
+# include <security/pam_appl.h>
+#endif
 
 #include "sudo.h"
 #include "sudo_auth.h"
 
+/* Only OpenPAM and Linux PAM use const qualifiers. */
+#if defined(_OPENPAM) || defined(__LIBPAM_VERSION)
+# define PAM_CONST	const
+#else
+# define PAM_CONST
+#endif
+
 #ifndef lint
-static const char rcsid[] = "$Sudo: pam.c,v 1.32 2003/03/15 20:37:44 millert Exp $";
+static const char rcsid[] = "$Sudo: pam.c,v 1.43 2004/06/28 14:51:50 millert Exp $";
 #endif /* lint */
 
 static int sudo_conv __P((int, PAM_CONST struct pam_message **,
@@ -77,6 +74,8 @@ static char *def_prompt;
 #define PAM_DATA_SILENT	0
 #endif
 
+static pam_handle_t *pamh;	/* global due to pam_prep_user() */
+
 int
 pam_init(pw, promptp, auth)
     struct passwd *pw;
@@ -84,19 +83,20 @@ pam_init(pw, promptp, auth)
     sudo_auth *auth;
 {
     static struct pam_conv pam_conv;
-    pam_handle_t *pamh;
+    static int pam_status;
 
     /* Initial PAM setup */
+    if (auth != NULL)
+	auth->data = (VOID *) &pam_status;
     pam_conv.conv = sudo_conv;
-    if (pam_start("sudo", pw->pw_name, &pam_conv, &pamh) != PAM_SUCCESS) {
-	log_error(USE_ERRNO|NO_EXIT|NO_MAIL, 
-	    "unable to initialize PAM");
+    pam_status = pam_start("sudo", pw->pw_name, &pam_conv, &pamh);
+    if (pam_status != PAM_SUCCESS) {
+	log_error(USE_ERRNO|NO_EXIT|NO_MAIL, "unable to initialize PAM");
 	return(AUTH_FATAL);
     }
     if (strcmp(user_tty, "unknown"))
 	(void) pam_set_item(pamh, PAM_TTY, user_tty);
 
-    auth->data = (VOID *) pamh;
     return(AUTH_SUCCESS);
 }
 
@@ -106,22 +106,51 @@ pam_verify(pw, prompt, auth)
     char *prompt;
     sudo_auth *auth;
 {
-    int error;
     const char *s;
-    pam_handle_t *pamh = (pam_handle_t *) auth->data;
+    int *pam_status = (int *) auth->data;
 
     def_prompt = prompt;	/* for sudo_conv */
 
     /* PAM_SILENT prevents the authentication service from generating output. */
-    error = pam_authenticate(pamh, PAM_SILENT);
-    switch (error) {
+    *pam_status = pam_authenticate(pamh, PAM_SILENT);
+    switch (*pam_status) {
 	case PAM_SUCCESS:
-	    return(AUTH_SUCCESS);
+	    *pam_status = pam_acct_mgmt(pamh, PAM_SILENT);
+	    switch (*pam_status) {
+		case PAM_SUCCESS:
+		    return(AUTH_SUCCESS);
+		case PAM_AUTH_ERR:
+		    log_error(NO_EXIT|NO_MAIL, "pam_acct_mgmt: %d",
+			*pam_status);
+		    return(AUTH_FAILURE);
+		case PAM_NEW_AUTHTOK_REQD:
+		    log_error(NO_EXIT|NO_MAIL, "%s, %s",
+			"Account or password is expired",
+			"reset your password and try again");
+		    *pam_status = pam_chauthtok(pamh,
+			PAM_CHANGE_EXPIRED_AUTHTOK);
+		    if (*pam_status == PAM_SUCCESS)
+			return(AUTH_SUCCESS);
+		    if ((s = pam_strerror(pamh, *pam_status)))
+			log_error(NO_EXIT|NO_MAIL, "pam_chauthtok: %s", s);
+		    return(AUTH_FAILURE);
+		case PAM_AUTHTOK_EXPIRED:
+		    log_error(NO_EXIT|NO_MAIL,
+			"Password expired, contact your system administrator");
+		    return(AUTH_FATAL);
+		case PAM_ACCT_EXPIRED:
+		    log_error(NO_EXIT|NO_MAIL, "%s %s",
+			"Account expired or PAM config lacks an \"account\"",
+			"section for sudo, contact your system administrator");
+		    return(AUTH_FATAL);
+	    }
+	    /* FALLTHROUGH */
 	case PAM_AUTH_ERR:
 	case PAM_MAXTRIES:
+	case PAM_PERM_DENIED:
 	    return(AUTH_FAILURE);
 	default:
-	    if ((s = pam_strerror(pamh, error)))
+	    if ((s = pam_strerror(pamh, *pam_status)))
 		log_error(NO_EXIT|NO_MAIL, "pam_authenticate: %s", s);
 	    return(AUTH_FATAL);
     }
@@ -132,47 +161,29 @@ pam_cleanup(pw, auth)
     struct passwd *pw;
     sudo_auth *auth;
 {
-    pam_handle_t *pamh = (pam_handle_t *) auth->data;
-    int status = PAM_DATA_SILENT;
+    int *pam_status = (int *) auth->data;
 
-    /* Convert AUTH_FOO -> PAM_FOO as best we can. */
-    /* XXX - store real value somewhere in auth->data and use it */
-    switch (auth->status) {
-	case AUTH_SUCCESS:
-	    status |= PAM_SUCCESS;
-	    break;
-	case AUTH_FAILURE:
-	    status |= PAM_AUTH_ERR;
-	    break;
-	case AUTH_FATAL:
-	default:
-	    status |= PAM_ABORT;
-	    break;
-    }
-
-    if (pam_end(pamh, status) == PAM_SUCCESS)
+    /* If successful, we can't close the session until pam_prep_user() */
+    if (auth->status == AUTH_SUCCESS)
 	return(AUTH_SUCCESS);
-    else
-	return(AUTH_FAILURE);
+
+    *pam_status = pam_end(pamh, *pam_status | PAM_DATA_SILENT);
+    return(*pam_status == PAM_SUCCESS ? AUTH_SUCCESS : AUTH_FAILURE);
 }
 
 int
 pam_prep_user(pw)
     struct passwd *pw;
 {
-    struct pam_conv pam_conv;
-    pam_handle_t *pamh;
+    if (pamh == NULL)
+	pam_init(pw, NULL, NULL);
 
-    /* We need to setup a new PAM session for the user we are changing *to*. */
-    pam_conv.conv = sudo_conv;
-    if (pam_start("sudo", pw->pw_name, &pam_conv, &pamh) != PAM_SUCCESS) {
-	log_error(USE_ERRNO|NO_EXIT|NO_MAIL, 
-	    "unable to initialize PAM");
-	return(AUTH_FATAL);
-    }
+    /*
+     * Set PAM_USER to the user we are changing *to* and
+     * set PAM_RUSER to the user we are coming *from*.
+     */
+    (void) pam_set_item(pamh, PAM_USER, pw->pw_name);
     (void) pam_set_item(pamh, PAM_RUSER, user_name);
-    if (strcmp(user_tty, "unknown"))
-	(void) pam_set_item(pamh, PAM_TTY, user_tty);
 
     /*
      * Set credentials (may include resource limits, device ownership, etc).
@@ -184,8 +195,8 @@ pam_prep_user(pw)
      */
     (void) pam_setcred(pamh, PAM_ESTABLISH_CRED);
 
-    if (pam_end(pamh, PAM_SUCCESS) == PAM_SUCCESS)
-	return(PAM_SUCCESS);
+    if (pam_end(pamh, PAM_SUCCESS | PAM_DATA_SILENT) == PAM_SUCCESS)
+	return(AUTH_SUCCESS);
     else
 	return(AUTH_FAILURE);
 }
@@ -201,34 +212,34 @@ sudo_conv(num_msg, msg, response, appdata_ptr)
     struct pam_response **response;
     VOID *appdata_ptr;
 {
-    volatile struct pam_response *pr;
+    struct pam_response *pr;
     PAM_CONST struct pam_message *pm;
     const char *p = def_prompt;
-    volatile char *pass;
+    char *pass;
     int n, flags;
     extern int nil_pw;
 
     if ((*response = malloc(num_msg * sizeof(struct pam_response))) == NULL)
 	return(PAM_CONV_ERR);
-    (void) memset(*response, 0, num_msg * sizeof(struct pam_response));
+    zero_bytes(*response, num_msg * sizeof(struct pam_response));
 
     for (pr = *response, pm = *msg, n = num_msg; n--; pr++, pm++) {
 	flags = tgetpass_flags;
 	switch (pm->msg_style) {
 	    case PAM_PROMPT_ECHO_ON:
-		flags |= TGP_ECHO;
+		SET(flags, TGP_ECHO);
 	    case PAM_PROMPT_ECHO_OFF:
 		/* Only override PAM prompt if it matches /^Password: ?/ */
 		if (strncmp(pm->msg, "Password:", 9) || (pm->msg[9] != '\0'
 		    && (pm->msg[9] != ' ' || pm->msg[10] != '\0')))
 		    p = pm->msg;
 		/* Read the password. */
-		pass = tgetpass(p, def_ival(I_PASSWD_TIMEOUT) * 60, flags);
+		pass = tgetpass(p, def_passwd_timeout * 60, flags);
 		pr->resp = estrdup(pass ? pass : "");
 		if (*pr->resp == '\0')
 		    nil_pw = 1;		/* empty password */
 		else
-		    memset(pass, 0, strlen(pass));
+		    zero_bytes(pass, strlen(pass));
 		break;
 	    case PAM_TEXT_INFO:
 		if (pm->msg)
@@ -244,13 +255,12 @@ sudo_conv(num_msg, msg, response, appdata_ptr)
 		/* Zero and free allocated memory and return an error. */
 		for (pr = *response, n = num_msg; n--; pr++) {
 		    if (pr->resp != NULL) {
-			(void) memset(pr->resp, 0, strlen(pr->resp));
+			zero_bytes(pr->resp, strlen(pr->resp));
 			free(pr->resp);
 			pr->resp = NULL;
 		    }
 		}
-		(void) memset(*response, 0,
-		    num_msg * sizeof(struct pam_response));
+		zero_bytes(*response, num_msg * sizeof(struct pam_response));
 		free(*response);
 		*response = NULL;
 		return(PAM_CONV_ERR);
