@@ -1,36 +1,18 @@
 /*
- * Copyright (c) 1999-2002 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 1999-2003 Todd C. Miller <Todd.Miller@courtesan.com>
  * Copyright (c) 2002 Michael Stroucken <michael@stroucken.org>
- * All rights reserved.
  *
- * This code is derived from software contributed by Michael Stroucken
- * <michael@stroucken.org>
+ * Permission to use, copy, modify, and distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *
- * 3. The name of the author may not be used to endorse or promote products
- *    derived from this software without specific prior written permission.
- *
- * 4. Products derived from this software may not be called "Sudo" nor
- *    may "Sudo" appear in their names without specific prior written
- *    permission from the author.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
- * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
- * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
- * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
- * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
- * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
+ * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
+ * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+ * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+ * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
  * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
  * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
@@ -63,6 +45,11 @@
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif /* HAVE_UNISTD_H */
+#ifdef HAVE_ERR_H
+# include <err.h>
+#else
+# include "emul/err.h"
+#endif /* HAVE_ERR_H */
 #include <pwd.h>
 
 /* Needed for SecurID v5.0 Authentication on UNIX */
@@ -74,7 +61,7 @@
 #include "sudo_auth.h"
 
 #ifndef lint
-static const char rcsid[] = "$Sudo: securid5.c,v 1.2 2002/05/20 20:51:23 millert Exp $";
+static const char rcsid[] = "$Sudo: securid5.c,v 1.6 2004/02/13 21:36:47 millert Exp $";
 #endif /* lint */
 
 /*
@@ -103,7 +90,7 @@ securid_init(pw, promptp, auth)
     if (AceInitialize() != SD_FALSE)
 	return(AUTH_SUCCESS);
 
-    fprintf(stderr, "Failed to initialise ACE API library.\n");
+    warnx("failed to initialise the ACE API library");
     return(AUTH_FATAL);
 }
 
@@ -131,7 +118,7 @@ securid_setup(pw, promptp, auth)
 
     /* Re-initialize SecurID every time. */
     if (SD_Init(sd) != ACM_OK) {
-	(void) fprintf(stderr, "%s: Cannot contact SecurID server\n", Argv[0]);
+	warnx("unable to contact the SecurID server");
 	return(AUTH_FATAL);
     }
 
@@ -140,19 +127,19 @@ securid_setup(pw, promptp, auth)
 
     switch (retval) {
         case ACE_UNDEFINED_USERNAME:
-		fprintf(stderr, "Invalid username length for SecurID\n");
+		warnx("invalid username length for SecurID");
 		return(AUTH_FATAL);
 
 	case ACE_ERR_INVALID_HANDLE:
-		fprintf(stderr, "Invalid Authentication Handle for SecurID\n");
+		warnx("invalid Authentication Handle for SecurID");
 		return(AUTH_FATAL);
 
 	case ACM_ACCESS_DENIED:
-		fprintf(stderr, "SecurID communication has failed\n");
+		warnx("SecurID communication failed");
 		return(AUTH_FATAL);
 
 	case ACM_OK:
-		fprintf(stderr, "User ID locked for SecurID Authentication\n");
+		warnx("User ID locked for SecurID Authentication");
 		return(AUTH_SUCCESS);
 	}
 }
@@ -179,22 +166,22 @@ securid_verify(pw, pass, auth)
     int rval;
 
     pass = (char *) tgetpass("Enter your PASSCODE: ",
-	def_ival(I_PASSWD_TIMEOUT) * 60, tgetpass_flags);
+	def_passwd_timeout * 60, tgetpass_flags);
 
     /* Have ACE verify password */
     switch (SD_Check(*sd, pass, pw->pw_name)) {
 	case ACE_UNDEFINED_PASSCODE:
-		fprintf(stderr, "Invalid passcode length for SecurID\n");
+		warnx("invalid passcode length for SecurID");
 		rval = AUTH_FATAL;
 		break;
 
 	case ACE_UNDEFINED_USERNAME:
-		fprintf(stderr, "Invalid username length for SecurID\n");
+		warnx("invalid username length for SecurID");
 		rval = AUTH_FATAL;
 		break;
 
 	case ACE_ERR_INVALID_HANDLE:
-		fprintf(stderr, "Invalid Authentication Handle for SecurID\n");
+		warnx("invalid Authentication Handle for SecurID");
 		rval = AUTH_FATAL;
 
 	case ACM_ACCESS_DENIED:
@@ -209,7 +196,7 @@ securid_verify(pw, pass, auth)
 !!! ATTENTION !!!\n\
 Wait for the token code to change, \n\
 then enter the new token code.\n", \
-		def_ival(I_PASSWD_TIMEOUT) * 60, tgetpass_flags);
+		def_passwd_timeout * 60, tgetpass_flags);
 
 		if (SD_Next(*sd, pass) == ACM_OK) {
 			rval = AUTH_SUCCESS;
